/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * University Copyright- Copyright (c) 1982, 1986, 1988
 * The Regents of the University of California
 * All Rights Reserved
 *
 * University Acknowledgment- Portions of this document are derived from
 * software developed by the University of California, Berkeley, and its
 * contributors.
 */

/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 *
 * Copyright 2016 RackTop Systems.
 */

/* Portions Copyright 2007 Jeremy Teo */
/* Portions Copyright 2006 Stephen P. Potter */

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>
#include <libgen.h>
#include <stdlib.h>
#include <errno.h>
#include <netdb.h>
#include <getopt.h>

#ifndef	TEXT_DOMAIN		/* should be defined by cc -D */
#define	TEXT_DOMAIN "SYS_TEST"	/* use this only if it wasn't */
#endif

static char *progname;

static void
usage(void)
{
	(void) fprintf(stderr, gettext("usage: %s [-fls|system_name]\n"),
		basename(progname));
	exit(1);
}

int
main(int argc, char *argv[])
{
	char	*nodename = NULL;
	char    c_hostname[MAXHOSTNAMELEN];
	int	optlet, optindex = 0;
	char	*optstring = "fls";
	int	fflag = 0, lflag = 0, sflag = 0;
	struct	option long_options[] = {
		{"fqdn",	no_argument,	0,	'f'	},
		{"long",	no_argument,	0,	'l'	},
		{"short",	no_argument,	0,	's'	},
		{NULL,		0,		0,	0	}
	};

	(void) setlocale(LC_ALL, "");
	(void) textdomain(TEXT_DOMAIN);

	progname = argv[0];

	opterr = 0;
	while ((optlet = getopt_long(argc, argv, optstring, long_options,
	    &optindex)) != -1) {
		switch (optlet) {
		case 'f':
			fflag++;
			break;
		case 'l':
			lflag++;
			break;
		case 's':
			sflag++;
			break;
		case '?':
			usage();
		}
	}

	/* No flag combinations allowed */
	if (fflag && (lflag || sflag))
		usage();
	if (lflag && (fflag || sflag))
		usage();
	if (sflag && (lflag || fflag))
		usage();

	/*
	 * if called with no options, just print out the hostname/nodename
	 */
	if (argc <= optind) {
		if (gethostname(c_hostname, sizeof (c_hostname)) < 0) {
			(void) fprintf(stderr,
			    gettext("%s: unable to obtain hostname\n"),
			    basename(progname));
			exit(1);
		}
		if (fflag || lflag) {
			struct addrinfo hints, *info;
			memset(&hints, 0, sizeof (struct addrinfo));
			hints.ai_socktype = SOCK_DGRAM;
			hints.ai_flags = AI_CANONNAME;
			if (getaddrinfo(c_hostname, NULL, &hints, &info) != 0) {
				(void) fprintf(stderr,
				    gettext("%s: unable to obtain fqdn\n"),
				    basename(progname));
				exit(1);
			}
			(void) fprintf(stdout, "%s\n", info->ai_canonname);
		} else {
			if (sflag) {
				char *p = strchr(c_hostname, '.');
				if (p != NULL)
					*p = '\0';
			}
			(void) fprintf(stdout, "%s\n", c_hostname);
		}
	} else {
		/*
		 * if called with an argument,
		 * we have to try to set the new hostname/nodename
		 */
		if (argc > optind + 1)
			usage();	/* too many arguments */
		if (fflag || lflag)
			usage();	/* can't set fqdn this way */
		if (sflag)
			usage();	/* can't set short name */
		nodename = argv[optind];
		if (sethostname(nodename, strlen(nodename)) < 0) {
			int err = errno;
			(void) fprintf(stderr,
			    gettext("%s: error in setting name: %s\n"),
			    basename(progname), strerror(err));
			exit(1);
		}
	}
	return (0);
}
